/*******************************************************************************
 * Copyright (c) 2008 The University of York.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     Dimitrios Kolovos - initial API and implementation
 ******************************************************************************/
package org.eclipse.epsilon.examples.standalone1.eol1;


import java.util.ArrayList;
import java.util.List;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.xmi.impl.EcoreResourceFactoryImpl;
import org.eclipse.epsilon.emc.emf.EmfUtil;
import org.eclipse.epsilon.eol.EolModule;
import org.eclipse.epsilon.eol.IEolExecutableModule;
import org.eclipse.epsilon.eol.models.IModel;
import org.eclipse.epsilon.examples.standalone1.EpsilonStandaloneExample;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.ui.IActionDelegate;

/**
 * This example demonstrates using the 
 * Epsilon Object Language, the core language
 * of Epsilon, in a stand-alone manner 
 * @author Dimitrios Kolovos
 */
public class EolStandaloneExample extends EpsilonStandaloneExample implements IActionDelegate{
	
	private String inputMetaModel;
	private String outputMetamodel;

	public EolStandaloneExample()
	{
		
	}
	
	@Override
	public IEolExecutableModule createModule() {
		return new EolModule();
	}

	
	@Override
	public List<IModel> getModels() throws Exception {
		
		inputMetaModel = "/Users/apple/Desktop/new eclipse/myPlugin/models/Tree.ecore";
		outputMetamodel = "/Users/apple/Desktop/new eclipse/myPlugin/models/Graph.ecore";
		List<IModel> models = new ArrayList<IModel>();
		URI emfURI = URI.createFileURI(inputMetaModel);
		URI emfURI1 = URI.createFileURI(outputMetamodel);
		
		EmfUtil.register(emfURI, EPackage.Registry.INSTANCE);
		EmfUtil.register(emfURI1, EPackage.Registry.INSTANCE);
		

		Resource.Factory.Registry.INSTANCE.getExtensionToFactoryMap().put("ecore", new EcoreResourceFactoryImpl ());
						
			ResourceSet resourceSet = new ResourceSetImpl(); 
			Resource resource1 = resourceSet.getResource(emfURI, true);
			EPackage inputPackage = (EPackage)resource1.getContents().get(0);
	 
			ResourceSet resourceSet1 = new ResourceSetImpl(); 
			Resource resource2 = resourceSet1.getResource(emfURI1, true);
			EPackage outputPackage = (EPackage)resource2.getContents().get(0);

		
			models.add(createEmfMetaModelByURI("IN_MM", "IN_MM", inputPackage.getNsURI()));

			models.add(createEmfMetaModelByURI("OUT_MM", "OUT_MM", outputPackage.getNsURI()));
					
		

		models.add(createEmfModelByURI("ATLOutput", "/Tree2Graph/ATLExpression.model",
				"http://www.eclipse.org/gmt/2005/ATL", true, true));
	return models;
	}

	@Override
	public String getSource() throws Exception {
		return "eol1/Demo.eol";
	}

	@Override
	public void postProcess() {
		
	}

	
	@Override
	public void run(IAction action) {
		// TODO Auto-generated method stub
		  try {
			new EolStandaloneExample().execute();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	@Override
	public void selectionChanged(IAction action, ISelection selection) {
		// TODO Auto-generated method stub
		
	}

	
}
